# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Basic example of clearing and drawing a pixel on a LED matrix display.
# This example and library is meant to work with Adafruit CircuitPython API.
# Author: Tony DiCola
# License: Public Domain

# Import all board pins.
import time
import board
import busio

# Import the HT16K33 LED matrix module.
from adafruit_ht16k33 import matrix


# Create the I2C interface.
i2c = busio.I2C(board.SCL, board.SDA)

# Create the matrix class.
# This creates a 16x8 matrix:
matrix = matrix.Matrix16x8(i2c)
# Or this creates a 16x8 matrix backpack:
# matrix = matrix.MatrixBackpack16x8(i2c)
# Or this creates a 8x8 matrix:
# matrix = matrix.Matrix8x8(i2c)
# Or this creates a 8x8 bicolor matrix:
# matrix = matrix.Matrix8x8x2(i2c)
# Finally you can optionally specify a custom I2C address of the HT16k33 like:
# matrix = matrix.Matrix16x8(i2c, address=0x70)

# Clear the matrix.
matrix.fill(0)

# Set a pixel in the origin 0, 0 position.
matrix[0, 0] = 1
# Set a pixel in the middle 8, 4 position.
matrix[8, 4] = 1
# Set a pixel in the opposite 15, 7 position.
matrix[15, 7] = 1

time.sleep(2)

# Draw a Smiley Face
matrix.fill(0)

for row in range(2, 6):
    matrix[row, 0] = 1
    matrix[row, 7] = 1

for column in range(2, 6):
    matrix[0, column] = 1
    matrix[7, column] = 1

matrix[1, 1] = 1
matrix[1, 6] = 1
matrix[6, 1] = 1
matrix[6, 6] = 1
matrix[2, 5] = 1
matrix[5, 5] = 1
matrix[2, 3] = 1
matrix[5, 3] = 1
matrix[3, 2] = 1
matrix[4, 2] = 1

# Move the Smiley Face Around
while True:
    for frame in range(0, 8):
        matrix.shift_right(True)
        time.sleep(0.05)
    for frame in range(0, 8):
        matrix.shift_down(True)
        time.sleep(0.05)
    for frame in range(0, 8):
        matrix.shift_left(True)
        time.sleep(0.05)
    for frame in range(0, 8):
        matrix.shift_up(True)
        time.sleep(0.05)



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Basic example of using the Bi-color 24 segment bargraph display.
# This example and library is meant to work with Adafruit CircuitPython API.
# Author: Carter Nelson
# License: Public Domain

import time

# Import board related modules
import board
import busio

# Import the Bicolor24 driver from the HT16K33 module
from adafruit_ht16k33.bargraph import Bicolor24

# Create the I2C interface
i2c = busio.I2C(board.SCL, board.SDA)

# Create the LED bargraph class.
bc24 = Bicolor24(i2c)

# Set individual segments of bargraph
bc24[0] = bc24.LED_RED
bc24[1] = bc24.LED_GREEN
bc24[2] = bc24.LED_YELLOW

time.sleep(2)

# Turn them all off
bc24.fill(bc24.LED_OFF)

# Turn them on in a loop
for i in range(24):
    bc24[i] = bc24.LED_RED
    time.sleep(0.1)
    bc24[i] = bc24.LED_OFF

time.sleep(1)

# Fill the entrire bargraph
bc24.fill(bc24.LED_GREEN)



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Basic example of setting digits on a LED segment display.
# This example and library is meant to work with Adafruit CircuitPython API.
# Author: Tony DiCola
# License: Public Domain

import time

# Import all board pins.
import board
import busio

# Import the HT16K33 LED segment module.
from adafruit_ht16k33 import segments

# Create the I2C interface.
i2c = busio.I2C(board.SCL, board.SDA)

# Create the LED segment class.
# This creates a 7 segment 4 character display:
display = segments.Seg7x4(i2c)
# Or this creates a 14 segment alphanumeric 4 character display:
# display = segments.Seg14x4(i2c)
# Or this creates a big 7 segment 4 character display
# display = segments.BigSeg7x4(i2c)
# Finally you can optionally specify a custom I2C address of the HT16k33 like:
# display = segments.Seg7x4(i2c, address=0x70)

# Clear the display.
display.fill(0)

# Can just print a number
display.print(42)
time.sleep(2)

# Or, can print a hexadecimal value
display.print_hex(0xFF23)
time.sleep(2)

# Or, print the time
display.print("12:30")
time.sleep(2)

display.colon = False

# Or, can set indivdual digits / characters
# Set the first character to '1':
display[0] = "1"
# Set the second character to '2':
display[1] = "2"
# Set the third character to 'A':
display[2] = "A"
# Set the forth character to 'B':
display[3] = "B"
time.sleep(2)

# Or, can even set the segments to make up characters
if isinstance(display, segments.Seg7x4):
    # 7-segment raw digits
    display.set_digit_raw(0, 0xFF)
    display.set_digit_raw(1, 0b11111111)
    display.set_digit_raw(2, 0x79)
    display.set_digit_raw(3, 0b01111001)
else:
    # 14-segment raw digits
    display.set_digit_raw(0, 0x2D3F)
    display.set_digit_raw(1, 0b0010110100111111)
    display.set_digit_raw(2, (0b00101101, 0b00111111))
    display.set_digit_raw(3, [0x2D, 0x3F])
time.sleep(2)

# Show a looping marquee
display.marquee("Deadbeef 192.168.100.102... ", 0.2)



# SPDX-FileCopyrightText: Radomir Dopieralski 2016 for Adafruit Industries
# SPDX-FileCopyrightText: Tony DiCola 2016 for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_ht16k33.ht16k33`
===========================

* Authors: Radomir Dopieralski & Tony DiCola for Adafruit Industries

"""

from adafruit_bus_device import i2c_device
from micropython import const

try:
    from typing import Optional
    from busio import I2C
except ImportError:
    pass

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_HT16K33.git"

_HT16K33_BLINK_CMD = const(0x80)
_HT16K33_BLINK_DISPLAYON = const(0x01)
_HT16K33_CMD_BRIGHTNESS = const(0xE0)
_HT16K33_OSCILATOR_ON = const(0x21)


class HT16K33:
    """
    The base class for all displays. Contains common methods.

    :param I2C i2c: The I2C bus object
    :param int address: The I2C addess of the HT16K33.
    :param bool auto_write: True if the display should immediately change when
        set. If False, `show` must be called explicitly.
    :param float brightness: 0.0 - 1.0 default brightness level.
    """

    def __init__(
        self,
        i2c: I2C,
        address: int = 0x70,
        auto_write: bool = True,
        brightness: float = 1.0,
    ) -> None:
        self.i2c_device = i2c_device.I2CDevice(i2c, address)
        self._temp = bytearray(1)
        self._buffer = bytearray(17)
        self._auto_write = auto_write
        self.fill(0)
        self._write_cmd(_HT16K33_OSCILATOR_ON)
        self._blink_rate = None
        self._brightness = None
        self.blink_rate = 0
        self.brightness = brightness

    def _write_cmd(self, byte: bytearray) -> None:
        self._temp[0] = byte
        with self.i2c_device:
            self.i2c_device.write(self._temp)

    @property
    def blink_rate(self) -> int:
        """The blink rate. Range 0-3."""
        return self._blink_rate

    @blink_rate.setter
    def blink_rate(self, rate: Optional[int] = None) -> None:
        if not 0 <= rate <= 3:
            raise ValueError("Blink rate must be an integer in the range: 0-3")
        rate = rate & 0x03
        self._blink_rate = rate
        self._write_cmd(_HT16K33_BLINK_CMD | _HT16K33_BLINK_DISPLAYON | rate << 1)

    @property
    def brightness(self) -> float:
        """The brightness. Range 0.0-1.0"""
        return self._brightness

    @brightness.setter
    def brightness(self, brightness: float) -> None:
        if not 0.0 <= brightness <= 1.0:
            raise ValueError(
                "Brightness must be a decimal number in the range: 0.0-1.0"
            )

        self._brightness = brightness
        xbright = round(15 * brightness)
        xbright = xbright & 0x0F
        self._write_cmd(_HT16K33_CMD_BRIGHTNESS | xbright)

    @property
    def auto_write(self) -> bool:
        """Auto write updates to the display."""
        return self._auto_write

    @auto_write.setter
    def auto_write(self, auto_write: bool) -> None:
        if isinstance(auto_write, bool):
            self._auto_write = auto_write
        else:
            raise ValueError("Must set to either True or False.")

    def show(self) -> None:
        """Refresh the display and show the changes."""
        with self.i2c_device:
            # Byte 0 is 0x00, address of LED data register. The remaining 16
            # bytes are the display register data to set.
            self.i2c_device.write(self._buffer)

    def fill(self, color: bool) -> None:
        """Fill the whole display with the given color.

        :param bool color: Whether to fill the display
        """

        fill = 0xFF if color else 0x00
        for i in range(16):
            self._buffer[i + 1] = fill
        if self._auto_write:
            self.show()

    def _pixel(self, x: int, y: int, color: Optional[bool] = None) -> Optional[bool]:
        addr = 2 * y + x // 8
        mask = 1 << x % 8
        if color is None:
            return bool(self._buffer[addr + 1] & mask)
        if color:
            # set the bit
            self._buffer[addr + 1] |= mask
        else:
            # clear the bit
            self._buffer[addr + 1] &= ~mask
        if self._auto_write:
            self.show()
        return None

    def _set_buffer(self, i: int, value: bool) -> None:
        self._buffer[i + 1] = value  # Offset by 1 to move past register address.

    def _get_buffer(self, i: int) -> bool:
        return self._buffer[i + 1]  # Offset by 1 to move past register address.



# SPDX-FileCopyrightText: Radomir Dopieralski 2016 for Adafruit Industries
# SPDX-FileCopyrightText: Tony DiCola 2016 for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
adafruit_ht16k33.matrix
=======================

"""
from adafruit_ht16k33.ht16k33 import HT16K33

try:
    from typing import Optional
    from PIL import Image
except ImportError:
    pass

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_HT16K33.git"


class Matrix8x8(HT16K33):
    """A single matrix."""

    _columns = 8
    _rows = 8

    def pixel(self, x: int, y: int, color: Optional[bool] = None) -> Optional[bool]:
        """Get or set the color of a given pixel.

        :param int x: The x coordinate of the pixel
        :param int y: The y coordinate of the pixel
        :param bool color: (Optional) The state to set the pixel
        :return: If ``color`` was not set, this returns the state of the pixel
        :rtype: bool
        """
        if not 0 <= x <= 7:
            return None
        if not 0 <= y <= 7:
            return None
        x = (x - 1) % 8
        return super()._pixel(x, y, color)

    def __getitem__(self, key: int) -> Optional[bool]:
        x, y = key
        return self.pixel(x, y)

    def __setitem__(self, key: int, value: bool) -> None:
        x, y = key
        self.pixel(x, y, value)

    # pylint: disable=too-many-branches
    def shift(self, x: int, y: int, rotate: bool = False) -> None:
        """
        Shift pixels by x and y

        :param int x: The x coordinate of the pixel
        :param int y: The y coordinate of the pixel
        :param bool rotate: (Optional) Rotate the shifted pixels to the left side (default=False)
        """
        auto_write = self.auto_write
        self._auto_write = False
        if x > 0:  # Shift Right
            for _ in range(x):
                for row in range(0, self.rows):
                    last_pixel = self[self.columns - 1, row] if rotate else 0
                    for col in range(self.columns - 1, 0, -1):
                        self[col, row] = self[col - 1, row]
                    self[0, row] = last_pixel
        elif x < 0:  # Shift Left
            for _ in range(-x):
                for row in range(0, self.rows):
                    last_pixel = self[0, row] if rotate else 0
                    for col in range(0, self.columns - 1):
                        self[col, row] = self[col + 1, row]
                    self[self.columns - 1, row] = last_pixel
        if y > 0:  # Shift Up
            for _ in range(y):
                for col in range(0, self.columns):
                    last_pixel = self[col, self.rows - 1] if rotate else 0
                    for row in range(self.rows - 1, 0, -1):
                        self[col, row] = self[col, row - 1]
                    self[col, 0] = last_pixel
        elif y < 0:  # Shift Down
            for _ in range(-y):
                for col in range(0, self.columns):
                    last_pixel = self[col, 0] if rotate else 0
                    for row in range(0, self.rows - 1):
                        self[col, row] = self[col, row + 1]
                    self[col, self.rows - 1] = last_pixel
        self._auto_write = auto_write
        if auto_write:
            self.show()

    # pylint: enable=too-many-branches

    def shift_right(self, rotate: bool = False) -> None:
        """
        Shift all pixels right

        :param rotate: (Optional) Rotate the shifted pixels to the left side (default=False)
        """
        self.shift(1, 0, rotate)

    def shift_left(self, rotate: bool = False) -> None:
        """
        Shift all pixels left

        :param rotate: (Optional) Rotate the shifted pixels to the right side (default=False)
        """
        self.shift(-1, 0, rotate)

    def shift_up(self, rotate: bool = False) -> None:
        """
        Shift all pixels up

        :param rotate: (Optional) Rotate the shifted pixels to bottom (default=False)
        """
        self.shift(0, 1, rotate)

    def shift_down(self, rotate: bool = False) -> None:
        """
        Shift all pixels down

        :param rotate: (Optional) Rotate the shifted pixels to top (default=False)
        """
        self.shift(0, -1, rotate)

    def image(self, img: Image) -> None:
        """Set buffer to value of Python Imaging Library image.  The image should
        be in 1 bit mode and a size equal to the display size.

        :param Image img: The image to show
        """

        imwidth, imheight = img.size
        if imwidth != self.columns or imheight != self.rows:
            raise ValueError(
                "Image must be same dimensions as display ({0}x{1}).".format(
                    self.columns, self.rows
                )
            )
        # Grab all the pixels from the image, faster than getpixel.
        pixels = img.convert("1").load()
        auto_write = self.auto_write
        self._auto_write = False
        # Iterate through the pixels
        for x in range(self.columns):  # yes this double loop is slow,
            for y in range(self.rows):  #  but these displays are small!
                self.pixel(x, y, pixels[(x, y)])
        self._auto_write = auto_write
        if self._auto_write:
            self.show()

    @property
    def columns(self) -> int:
        """Read-only property for number of columns"""
        return self._columns

    @property
    def rows(self) -> int:
        """Read-only property for number of rows"""
        return self._rows


class Matrix16x8(Matrix8x8):
    """The matrix wing."""

    _columns = 16

    def pixel(self, x: int, y: int, color: Optional[bool] = None) -> Optional[bool]:
        """Get or set the color of a given pixel.

        :param int x: The x coordinate of the pixel
        :param int y: The y coordinate of the pixel
        :param bool color: (Optional) The state to set the pixel
        :return: If ``color`` was not set, this returns the state of the pixel
        :rtype: bool
        """

        if not 0 <= x <= 15:
            return None
        if not 0 <= y <= 7:
            return None
        if x >= 8:
            x -= 8
            y += 8
        return super()._pixel(y, x, color)  # pylint: disable=arguments-out-of-order


class MatrixBackpack16x8(Matrix16x8):
    """A double matrix backpack."""

    def pixel(self, x: int, y: int, color: Optional[bool] = None) -> Optional[bool]:
        """Get or set the color of a given pixel.

        :param int x: The x coordinate of the pixel
        :param int y: The y coordinate of the pixel
        :param bool color: (Optional) The state to set the pixel
        :return: If ``color`` was not set, this returns the state of the pixel
        :rtype: bool
        """

        if not 0 <= x <= 15:
            return None
        if not 0 <= y <= 7:
            return None
        return super()._pixel(x, y, color)


class Matrix8x8x2(Matrix8x8):
    """A bi-color matrix."""

    LED_OFF = 0
    LED_RED = 1
    LED_GREEN = 2
    LED_YELLOW = 3

    def pixel(self, x: int, y: int, color: Optional[bool] = None) -> Optional[bool]:
        """Get or set the color of a given pixel.

        :param int x: The x coordinate of the pixel
        :param int y: The y coordinate of the pixel
        :param bool color: (Optional) The state to set the pixel
        :return: If ``color`` was not set, this returns the state of the pixel
        :rtype: bool
        """
        if not 0 <= x <= 7:
            return None
        if not 0 <= y <= 7:
            return None
        if color is not None:
            super()._pixel(y, x, (color >> 1) & 0x01)
            super()._pixel(y + 8, x, (color & 0x01))
        else:
            return super()._pixel(y, x) | super()._pixel(y + 8, x) << 1
        return None

    def fill(self, color: bool) -> None:
        """Fill the whole display with the given color.

        :param bool color: Whether to fill the display
        """

        fill1 = 0xFF if color & 0x01 else 0x00
        fill2 = 0xFF if color & 0x02 else 0x00
        for i in range(8):
            self._set_buffer(i * 2, fill1)
            self._set_buffer(i * 2 + 1, fill2)
        if self._auto_write:
            self.show()

    def image(self, img: Image) -> None:
        """Set buffer to value of Python Imaging Library image.  The image should
        be a size equal to the display size.

        :param Image img: The image to show
        """

        imwidth, imheight = img.size
        if imwidth != self.columns or imheight != self.rows:
            raise ValueError(
                "Image must be same dimensions as display ({0}x{1}).".format(
                    self.columns, self.rows
                )
            )
        # Grab all the pixels from the image, faster than getpixel.
        pixels = img.convert("RGB").load()
        auto_write = self.auto_write
        self._auto_write = False
        # Iterate through the pixels
        for x in range(self.columns):  # yes this double loop is slow,
            for y in range(self.rows):  #  but these displays are small!
                if pixels[(x, y)] == (255, 0, 0):
                    self.pixel(x, y, self.LED_RED)
                elif pixels[(x, y)] == (0, 255, 0):
                    self.pixel(x, y, self.LED_GREEN)
                elif pixels[(x, y)] == (255, 255, 0):
                    self.pixel(x, y, self.LED_YELLOW)
                else:
                    # Unknown color, default to LED off.
                    self.pixel(x, y, self.LED_OFF)
        self._auto_write = auto_write
        if self._auto_write:
            self.show()



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Basic example of using FrameBuffer to create and scroll text on the matrix.

# Requires: adafruit_framebuf

import board
import busio
import adafruit_framebuf

# Import the HT16K33 LED matrix module.
from adafruit_ht16k33 import matrix

# Create the I2C interface.
i2c = busio.I2C(board.SCL, board.SDA)

# Create the matrix class.
# This creates a 16x8 matrix:
matrix = matrix.Matrix16x8(i2c)

# Low brightness so it's easier to look at
matrix.brightness = 0

# Clear the matrix.
matrix.fill(0)

text_to_show = "Hello Blinka"

# Create a framebuffer for our display
buf = bytearray(16)  # 1 bytes tall x 16 wide = 16 bytes
fb = adafruit_framebuf.FrameBuffer(buf, 16, 8, adafruit_framebuf.MVLSB)


while True:
    for i in range(len(text_to_show) * 8):
        fb.fill(0)
        fb.text(text_to_show, -i + 16, 0, color=1)
        # turn all LEDs off
        matrix.fill(0)
        for x in range(16):
            # using the FrameBuffer text result
            bite = buf[x]
            for y in range(8):
                bit = 1 << y & bite
                # if bit > 0 then set the pixel brightness
                if bit:
                    matrix[16 - x, y + 1] = 1



# SPDX-FileCopyrightText: 2018 Carter Nelson  for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_ht16k33.bargraph`
===========================

* Authors: Carter Nelson for Adafruit Industries

"""

from adafruit_ht16k33.ht16k33 import HT16K33

try:
    from typing import Optional
except ImportError:
    pass

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_HT16K33.git"


class Bicolor24(HT16K33):
    """Bi-color 24-bar bargraph display."""

    LED_OFF = 0
    LED_RED = 1
    LED_GREEN = 2
    LED_YELLOW = 3

    def __getitem__(self, key: int) -> Optional[bool]:
        # map to HT16K33 row (x) and column (y), see schematic
        x = key % 4 + 4 * (key // 12)
        y = key // 4 - 3 * (key // 12)
        # construct the color value and return it
        return self._pixel(x, y) | self._pixel(x + 8, y) << 1

    def __setitem__(self, key: int, value: bool) -> None:
        # map to HT16K33 row (x) and column (y), see schematic
        x = key % 4 + 4 * (key // 12)
        y = key // 4 - 3 * (key // 12)
        # conditionally turn on red LED
        self._pixel(x, y, value & 0x01)
        # conditionally turn on green LED
        self._pixel(x + 8, y, value >> 1)

    def fill(self, color: bool) -> None:
        """Fill the whole display with the given color.

        :param bool color: Whether to fill the display
        """

        what_it_was = self.auto_write
        self.auto_write = False
        for i in range(24):
            self[i] = color
        self.show()
        self.auto_write = what_it_was



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

"""
    Test script for display animations on an HT16K33 with alphanumeric display

    The display must be initialized with auto_write=False.
"""

from time import sleep
import board
from adafruit_ht16k33.segments import Seg14x4
from adafruit_ht16k33.animations import Animation

#   Initialize the I2C bus
i2c = board.I2C()  # uses board.SCL and board.SDA
display = Seg14x4(i2c, auto_write=False)
#   Brightness of the display (0 to 15)
display.brightness = 0.3

ani = Animation(display)

try:
    text = "Init"

    display.fill(1)
    display.show()
    sleep(1)
    display.fill(0)
    display.show()

    display.print(text)
    display.show()
    sleep(2)
    display.fill(0)
    display.show()
    sleep(1)

    ani.count_down()
    sleep(0.2)

    text = "Go!!"

    display.print(text)
    display.show()
    sleep(1.5)
    display.fill(0)
    display.show()
    sleep(0.5)
    print()

    while True:
        #   Arrow
        print("Arrow")
        ani.animate([0, 1, 2], [192], 0.1)
        ani.animate([3], [2368], 0.1)
        sleep(1.0)
        display.fill(0)
        sleep(1.0)

        #   Flying
        print("Flying")
        cyc = 0

        while cyc < 5:
            ani.animate([0], [1280, 192, 10240, 192], 0.2)

            cyc += 1

        ani.animate([0], [0])
        sleep(1.0)
        display.fill(0)
        sleep(1.0)

        #   Chase forward and reverse.
        print("Chase forward and reverse")
        ani.chase_forward_and_reverse(0.01, 5)
        sleep(1.0)
        display.fill(0)
        sleep(1.0)

        #   Testing writing to more than one segment simultaneously
        print("Prelude to Spinners")
        ani.prelude_to_spinners(0.1, 5)
        sleep(1.0)
        display.fill(0)
        display.show()
        sleep(1.0)

        print("Spinners")
        ani.spinners(0.1, 20)
        sleep(1.0)
        display.fill(0)
        display.show()
        sleep(1.0)

        print("Enclosed Spinners")
        ani.enclosed_spinners(0.1, 20)
        sleep(1.0)
        display.fill(0)
        display.show()
        sleep(1.0)

        print()
except KeyboardInterrupt:
    display.fill(0)
    display.show()



# SPDX-FileCopyrightText: Radomir Dopieralski 2016  for Adafruit Industries
# SPDX-FileCopyrightText: Tony DiCola 2016 for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
adafruit_ht16k33.segments
=========================
"""

from time import sleep
from adafruit_ht16k33.ht16k33 import HT16K33

try:
    from typing import Union, List, Tuple
    from busio import I2C
except ImportError:
    pass

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_HT16K33.git"

# fmt: off
CHARS = (
    0b00000000, 0b00000000, #
    0b01000000, 0b00000110, # !
    0b00000010, 0b00100000, # "
    0b00010010, 0b11001110, # #
    0b00010010, 0b11101101, # $
    0b00001100, 0b00100100, # %
    0b00100011, 0b01011101, # &
    0b00000100, 0b00000000, # '
    0b00100100, 0b00000000, # (
    0b00001001, 0b00000000, # )
    0b00111111, 0b11000000, # *
    0b00010010, 0b11000000, # +
    0b00001000, 0b00000000, # ,
    0b00000000, 0b11000000, # -
    0b00000000, 0b00000000, # .
    0b00001100, 0b00000000, # /
    0b00001100, 0b00111111, # 0
    0b00000000, 0b00000110, # 1
    0b00000000, 0b11011011, # 2
    0b00000000, 0b10001111, # 3
    0b00000000, 0b11100110, # 4
    0b00100000, 0b01101001, # 5
    0b00000000, 0b11111101, # 6
    0b00000000, 0b00000111, # 7
    0b00000000, 0b11111111, # 8
    0b00000000, 0b11101111, # 9
    0b00010010, 0b00000000, # :
    0b00001010, 0b00000000, # ;
    0b00100100, 0b01000000, # <
    0b00000000, 0b11001000, # =
    0b00001001, 0b10000000, # >
    0b01100000, 0b10100011, # ?
    0b00000010, 0b10111011, # @
    0b00000000, 0b11110111, # A
    0b00010010, 0b10001111, # B
    0b00000000, 0b00111001, # C
    0b00010010, 0b00001111, # D
    0b00000000, 0b11111001, # E
    0b00000000, 0b01110001, # F
    0b00000000, 0b10111101, # G
    0b00000000, 0b11110110, # H
    0b00010010, 0b00000000, # I
    0b00000000, 0b00011110, # J
    0b00100100, 0b01110000, # K
    0b00000000, 0b00111000, # L
    0b00000101, 0b00110110, # M
    0b00100001, 0b00110110, # N
    0b00000000, 0b00111111, # O
    0b00000000, 0b11110011, # P
    0b00100000, 0b00111111, # Q
    0b00100000, 0b11110011, # R
    0b00000000, 0b11101101, # S
    0b00010010, 0b00000001, # T
    0b00000000, 0b00111110, # U
    0b00001100, 0b00110000, # V
    0b00101000, 0b00110110, # W
    0b00101101, 0b00000000, # X
    0b00010101, 0b00000000, # Y
    0b00001100, 0b00001001, # Z
    0b00000000, 0b00111001, # [
    0b00100001, 0b00000000, # \
    0b00000000, 0b00001111, # ]
    0b00001100, 0b00000011, # ^
    0b00000000, 0b00001000, # _
    0b00000001, 0b00000000, # `
    0b00010000, 0b01011000, # a
    0b00100000, 0b01111000, # b
    0b00000000, 0b11011000, # c
    0b00001000, 0b10001110, # d
    0b00001000, 0b01011000, # e
    0b00000000, 0b01110001, # f
    0b00000100, 0b10001110, # g
    0b00010000, 0b01110000, # h
    0b00010000, 0b00000000, # i
    0b00000000, 0b00001110, # j
    0b00110110, 0b00000000, # k
    0b00000000, 0b00110000, # l
    0b00010000, 0b11010100, # m
    0b00010000, 0b01010000, # n
    0b00000000, 0b11011100, # o
    0b00000001, 0b01110000, # p
    0b00000100, 0b10000110, # q
    0b00000000, 0b01010000, # r
    0b00100000, 0b10001000, # s
    0b00000000, 0b01111000, # t
    0b00000000, 0b00011100, # u
    0b00100000, 0b00000100, # v
    0b00101000, 0b00010100, # w
    0b00101000, 0b11000000, # x
    0b00100000, 0b00001100, # y
    0b00001000, 0b01001000, # z
    0b00001001, 0b01001001, # {
    0b00010010, 0b00000000, # |
    0b00100100, 0b10001001, # }
    0b00000101, 0b00100000, # ~
    0b00111111, 0b11111111,
)
# fmt: on
NUMBERS = (
    0x3F,  # 0
    0x06,  # 1
    0x5B,  # 2
    0x4F,  # 3
    0x66,  # 4
    0x6D,  # 5
    0x7D,  # 6
    0x07,  # 7
    0x7F,  # 8
    0x6F,  # 9
    0x77,  # a
    0x7C,  # b
    0x39,  # C
    0x5E,  # d
    0x79,  # E
    0x71,  # F
    0x40,  # -
)


class Seg14x4(HT16K33):
    """Alpha-numeric, 14-segment display."""

    def print(self, value: Union[str, int, float], decimal: int = 0) -> None:
        """Print the value to the display.

        :param value: The value to print
        :type value: str, int, or float
        :param int decimal: The number of decimal places for a floating point
            number if decimal is greater than zero, or the input number is an
            integer if decimal is zero.
        """

        if isinstance(value, (str)):
            self._text(value)
        elif isinstance(value, (int, float)):
            self._number(value, decimal)
        else:
            raise ValueError("Unsupported display value type: {}".format(type(value)))
        if self._auto_write:
            self.show()

    def print_hex(self, value: int) -> None:
        """Print the value as a hexidecimal string to the display.

        :param int value: The number to print
        """

        if isinstance(value, int):
            self.print("{0:X}".format(value))
        else:
            self.print(value)

    def __setitem__(self, key: str, value: Union[str, int, float]) -> None:
        self._put(value, key)
        if self._auto_write:
            self.show()

    def scroll(self, count: int = 1) -> None:
        """Scroll the display by specified number of places.

        :param int count: The number of places to scroll
        """

        if count >= 0:
            offset = 0
        else:
            offset = 2
        for i in range(6):
            self._set_buffer(i + offset, self._get_buffer(i + 2 * count))

    def _put(self, char: str, index: int = 0) -> None:
        """Put a character at the specified place."""
        if not 0 <= index <= 3:
            return
        if not 32 <= ord(char) <= 127:
            return
        if char == ".":
            self._set_buffer(
                index * 2 + 1, self._get_buffer(index * 2 + 1) | 0b01000000
            )
            return
        character = ord(char) * 2 - 64
        self._set_buffer(index * 2, CHARS[1 + character])
        self._set_buffer(index * 2 + 1, CHARS[character])

    def _push(self, char: str) -> None:
        """Scroll the display and add a character at the end."""
        if char != "." or self._get_buffer(7) & 0b01000000:
            self.scroll()
            self._put(" ", 3)
        self._put(char, 3)

    def _text(self, text: str) -> None:
        """Display the specified text."""
        for character in text:
            self._push(character)

    def _number(self, number: Union[int, float], decimal: int = 0) -> str:
        """
        Display a floating point or integer number on the Adafruit HT16K33 based displays

        Param: number - The floating point or integer number to be displayed, which must be
                in the range 0 (zero) to 9999 for integers and floating point or integer numbers
                and between 0.0 and 999.0 or 99.00 or 9.000 for floating point numbers.
        Param: decimal - The number of decimal places for a floating point number if decimal
                is greater than zero, or the input number is an integer if decimal is zero.

        Returns: The output text string to be displayed.
        """

        auto_write = self._auto_write
        self._auto_write = False
        stnum = str(number)
        dot = stnum.find(".")

        if (len(stnum) > 5) or ((len(stnum) > 4) and (dot < 0)):
            raise ValueError(
                "Input overflow - {0} is too large for the display!".format(number)
            )

        if dot < 0:
            # No decimal point (Integer)
            places = len(stnum)
        else:
            places = len(stnum[:dot])

        if places <= 0 < decimal:
            self.fill(False)
            places = 4

            if "." in stnum:
                places += 1

        # Set decimal places, if number of decimal places is specified (decimal > 0)
        if places > 0 < decimal < len(stnum[places:]) and dot > 0:
            txt = stnum[: dot + decimal + 1]
        elif places > 0:
            txt = stnum[:places]

        if len(txt) > 5:
            raise ValueError("Output string ('{0}') is too long!".format(txt))

        self._text(txt)
        self._auto_write = auto_write

        return txt

    def set_digit_raw(self, index: int, bitmask: Union[int, List, Tuple]) -> None:
        """Set digit at position to raw bitmask value. Position should be a value
        of 0 to 3 with 0 being the left most character on the display.

        :param int index: The index of the display to set
        :param bitmask: A 2 byte number corresponding to the segments to set
        :type bitmask: int, or a list/tuple of bool
        """
        if not isinstance(index, int) or not 0 <= index <= 3:
            raise ValueError("Index value must be an integer in the range: 0-3")

        if isinstance(bitmask, (tuple, list)):
            bitmask = ((bitmask[0] & 0xFF) << 8) | (bitmask[1] & 0xFF)

        # Use only the valid potion of bitmask
        bitmask &= 0xFFFF

        # Set the digit bitmask value at the appropriate position.
        self._set_buffer(index * 2, bitmask & 0xFF)
        self._set_buffer(index * 2 + 1, (bitmask >> 8) & 0xFF)

        if self._auto_write:
            self.show()

    def marquee(self, text: str, delay: float = 0.25, loop: bool = True) -> None:
        """
        Automatically scroll the text at the specified delay between characters

        :param str text: The text to display
        :param float delay: (optional) The delay in seconds to pause before scrolling
                            to the next character (default=0.25)
        :param bool loop: (optional) Whether to endlessly loop the text (default=True)

        """
        if isinstance(text, str):
            self.fill(False)
            if loop:
                while True:
                    self._scroll_marquee(text, delay)
            else:
                self._scroll_marquee(text, delay)

    def _scroll_marquee(self, text: str, delay: float) -> None:
        """Scroll through the text string once using the delay"""
        char_is_dot = False
        for character in text:
            self.print(character)
            # Add delay if character is not a dot or more than 2 in a row
            if character != "." or char_is_dot:
                sleep(delay)
            char_is_dot = character == "."
            self.show()


class Seg7x4(Seg14x4):
    """Numeric 7-segment display. It has the same methods as the alphanumeric display, but only
    supports displaying a limited set of characters.

    :param I2C i2c: The I2C bus object
    :param int address: The I2C address for the display
    :param bool auto_write: True if the display should immediately change when set. If False,
        `show` must be called explicitly.
    """

    POSITIONS = (0, 2, 6, 8)  #  The positions of characters.

    def __init__(self, i2c: I2C, address: int = 0x70, auto_write: bool = True) -> None:
        super().__init__(i2c, address, auto_write)
        # Use colon for controling two-dots indicator at the center (index 0)
        self._colon = Colon(self)

    def scroll(self, count: int = 1) -> None:
        """Scroll the display by specified number of places.

        :param int count: The number of places to scroll
        """

        if count >= 0:
            offset = 0
        else:
            offset = 1
        for i in range(3):
            self._set_buffer(
                self.POSITIONS[i + offset], self._get_buffer(self.POSITIONS[i + count])
            )

    def _push(self, char: str) -> None:
        """Scroll the display and add a character at the end."""
        if char in ":;":
            self._put(char)
        else:
            if char != "." or self._get_buffer(self.POSITIONS[3]) & 0b10000000:
                self.scroll()
                self._put(" ", 3)
            self._put(char, 3)

    def _put(self, char: str, index: int = 0) -> None:
        """Put a character at the specified place."""
        # pylint: disable=too-many-return-statements
        if not 0 <= index <= 3:
            return
        char = char.lower()
        index = self.POSITIONS[index]
        if char == ".":
            self._set_buffer(index, self._get_buffer(index) | 0b10000000)
            return
        if char in "abcdef":
            character = ord(char) - 97 + 10
        elif char == "-":
            character = 16
        elif char in "0123456789":
            character = ord(char) - 48
        elif char == " ":
            self._set_buffer(index, 0x00)
            return
        elif char == ":":
            self._set_buffer(4, 0x02)
            return
        elif char == ";":
            self._set_buffer(4, 0x00)
            return
        elif char in "lL":
            self._set_buffer(index, 0b00111000)
            return
        elif char in "oO":
            self._set_buffer(index, 0b00111111)
            return
        else:
            return
        self._set_buffer(index, NUMBERS[character])

    def set_digit_raw(self, index: int, bitmask: int) -> None:
        """Set digit at position to raw bitmask value. Position should be a value
        of 0 to 3 with 0 being the left most digit on the display.

        :param int index: The index of the display to set
        :param bitmask: A 2 byte number corresponding to the segments to set
        :type bitmask: int, or a list/tuple of bool
        """

        if not isinstance(index, int) or not 0 <= index <= 3:
            raise ValueError("Index value must be an integer in the range: 0-3")

        # Set the digit bitmask value at the appropriate position.
        self._set_buffer(self.POSITIONS[index], bitmask & 0xFF)

        if self._auto_write:
            self.show()

    @property
    def colon(self) -> bool:
        """Simplified colon accessor"""
        return self._colon[0]

    @colon.setter
    def colon(self, turn_on: bool) -> None:
        self._colon[0] = turn_on


class BigSeg7x4(Seg7x4):
    """Numeric 7-segment display. It has the same methods as the alphanumeric display, but only
    supports displaying a limited set of characters.

    :param I2C i2c: The I2C bus object
    :param int address: The I2C address for the display
    :param bool auto_write: True if the display should immediately change when set. If False,
        `show` must be called explicitly.
    """

    def __init__(self, i2c: I2C, address: int = 0x70, auto_write: bool = True) -> None:
        super().__init__(i2c, address, auto_write)
        # Use colon for controling two-dots indicator at the center (index 0)
        # or the two-dots indicators at the left (index 1)
        self.colon = Colon(self, 2)

    def _setindicator(self, index: int, value: bool) -> None:
        """Set side LEDs (dots)
        Index is as follow :
        * 0 : two dots at the center
        * 1 : top-left dot
        * 2 : bottom-left dot
        * 3 : right dot (also ampm indicator)
        """
        bitmask = 1 << (index + 1)
        current = self._get_buffer(0x04)
        if value:
            self._set_buffer(0x04, current | bitmask)
        else:
            self._set_buffer(0x04, current & ~bitmask)
        if self._auto_write:
            self.show()

    def _getindicator(self, index: int) -> int:
        """Get side LEDs (dots)
        See setindicator() for indexes
        """
        bitmask = 1 << (index + 1)
        return self._get_buffer(0x04) & bitmask

    @property
    def top_left_dot(self) -> bool:
        """The top-left dot indicator."""
        return bool(self._getindicator(1))

    @top_left_dot.setter
    def top_left_dot(self, value: bool) -> None:
        self._setindicator(1, value)

    @property
    def bottom_left_dot(self) -> bool:
        """The bottom-left dot indicator."""
        return bool(self._getindicator(2))

    @bottom_left_dot.setter
    def bottom_left_dot(self, value: bool) -> None:
        self._setindicator(2, value)

    @property
    def ampm(self) -> bool:
        """The AM/PM indicator."""
        return bool(self._getindicator(3))

    @ampm.setter
    def ampm(self, value: bool) -> None:
        self._setindicator(3, value)


class Colon:
    """Helper class for controlling the colons. Not intended for direct use."""

    # pylint: disable=protected-access

    MASKS = (0x02, 0x0C)

    def __init__(self, disp: Seg7x4, num_of_colons: int = 1) -> None:
        self._disp = disp
        self._num_of_colons = num_of_colons

    def __setitem__(self, key: int, value: bool) -> None:
        if key > self._num_of_colons - 1:
            raise ValueError("Trying to set a non-existent colon.")
        current = self._disp._get_buffer(0x04)
        if value:
            self._disp._set_buffer(0x04, current | self.MASKS[key])
        else:
            self._disp._set_buffer(0x04, current & ~self.MASKS[key])
        if self._disp.auto_write:
            self._disp.show()

    def __getitem__(self, key: int) -> bool:
        if key > self._num_of_colons - 1:
            raise ValueError("Trying to access a non-existent colon.")
        return bool(self._disp._get_buffer(0x04) & self.MASKS[key])



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT


"""
`adafruit_ht16k33.animations`
==============================

* Authors: ladyada

Test script for display animations on an HT16K33 with alphanumeric display

The display must be initialized with auto_write=False.

"""

from time import sleep

N = 16384
M = 8192
L = 4096
K = 2048
J = 1024
I = 512
H = 256
G2 = 128
G1 = 64
F = 32
E = 16
D = 8
C = 4
B = 2
A = 1


# pylint: disable=invalid-name


class Animation:
    """Animation class for the htk33
    Main driver for all alphanumeric display animations (WIP!!!)

    :param display: HTK33 Display object


    """

    def __init__(self, display):

        self._display = display

    def animate(self, digits, bitmasks, delay=0.2, auto_write=True):
        """Animate function


        :param digits: a list of the digits to write to, in order, like [0, 1, 3]. The digits are
         0 to 3 starting at the left most digit.
        :param bitmasks: a list of the bitmasks to write, in sequence, to the specified digits.
        :param delay: The delay, in seconds (or fractions of), between writing bitmasks to a digit.
        :param auto_write: Whether to actually write to the display immediately or not.


        """

        if not isinstance(digits, list):
            raise ValueError("The first parameter MUST be a list!")
        if not isinstance(bitmasks, list):
            raise ValueError("The second parameter MUST be a list!")
        if delay < 0:
            raise ValueError("The delay between frames must be positive!")
        for dig in digits:
            if not 0 <= dig <= 3:
                raise ValueError("Digit value must be an integer in the range: 0-3")

            for bits in bitmasks:
                if not 0 <= bits <= 0xFFFF:
                    raise ValueError(
                        "Bitmask value must be an integer in the range: 0-65535"
                    )

                self._display.set_digit_raw(dig, bits)

                if auto_write:
                    self._display.show()
                    sleep(delay)

    def chase_forward_and_reverse(self, delay=0.2, cycles=5):
        """Chase Forward and Reverse Animation"""

        cy = 0

        while cy < cycles:
            self.animate([0, 1, 2, 3], [A, 0], delay)
            self.animate([3], [B, C, D, 0], delay)
            self.animate([2, 1, 0], [D, 0], delay)
            self.animate([0], [E, F, H, G2, 0], delay)
            self.animate([1, 2], [G1, G2, 0], delay)
            self.animate([3], [G1, J, A, 0], delay)
            self.animate([2, 1], [A, 0], delay)
            self.animate([0], [A, F, E, D, 0], delay)
            self.animate([1, 2], [D, 0], delay)
            self.animate([3], [D, C, B, J, G1, 0], delay)
            self.animate([2, 1], [G2, G1, 0], delay)
            self.animate([0], [H, 0], delay)

            cy += 1

    def prelude_to_spinners(self, delay=0.2, cycles=5):
        """Prelude to Spinners Animation"""

        cy = 0
        auto_write = False

        while cy < cycles:
            self.animate([1, 2], [A], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0, 3], [A], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + F], 0, auto_write)
            self.animate([3], [A + B], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + E + F], 0, auto_write)
            self.animate([3], [A + B + C], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + D + E + F], 0, auto_write)
            self.animate([3], [A + B + C + D], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([1], [A + D], 0, auto_write)
            self.animate([2], [A + D], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([1], [A + D + M], 0, auto_write)
            self.animate([2], [A + D + K], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([1], [A + D + M + H], 0, auto_write)
            self.animate([2], [A + D + K + J], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + E + F + J + D], 0, auto_write)
            self.animate([3], [A + B + C + H + D], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + E + F + J + K + D], 0, auto_write)
            self.animate([3], [A + B + C + H + M + D], 0, auto_write)
            self._display.show()
            sleep(delay)

            self._display.fill(0)
            self._display.show()
            sleep(delay)

            cy += 1

    def spinners(self, delay=0.2, cycles=5):
        """Spinners Animation"""

        cy = 0
        auto_write = False

        while cy < cycles:
            self.animate([0], [H + M], 0, auto_write)
            self.animate([1], [J + K], 0, auto_write)
            self.animate([2], [H + M], 0, auto_write)
            self.animate([3], [J + K], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [G1 + G2], 0, auto_write)
            self.animate([1], [G1 + G2], 0, auto_write)
            self.animate([2], [G1 + G2], 0, auto_write)
            self.animate([3], [G1 + G2], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [J + K], 0, auto_write)
            self.animate([1], [H + M], 0, auto_write)
            self.animate([2], [J + K], 0, auto_write)
            self.animate([3], [H + M], 0, auto_write)
            self._display.show()
            sleep(delay)

            cy += 1

        self._display.fill(0)

    def enclosed_spinners(self, delay=0.2, cycles=5):
        """Enclosed Spinner Animation"""
        cy = 0
        auto_write = False

        while cy < cycles:
            self.animate([0], [A + D + E + F + H + M], 0, auto_write)
            self.animate([1], [A + D + J + K], 0, auto_write)
            self.animate([2], [A + D + H + M], 0, auto_write)
            self.animate([3], [A + B + C + D + J + K], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + D + E + F + G1 + G2], 0, auto_write)
            self.animate([1], [A + D + G1 + G2], 0, auto_write)
            self.animate([2], [A + D + G1 + G2], 0, auto_write)
            self.animate([3], [A + B + C + D + G1 + G2], 0, auto_write)
            self._display.show()
            sleep(delay)

            self.animate([0], [A + D + E + F + J + K], 0, auto_write)
            self.animate([1], [A + D + H + M], 0, auto_write)
            self.animate([2], [A + D + J + K], 0, auto_write)
            self.animate([3], [A + B + C + D + H + M], 0, auto_write)
            self._display.show()
            sleep(delay)

            cy += 1

        self._display.fill(0)

    def count_down(self):
        """Countdown Method"""
        auto_write = False
        numbers = [
            [A + B + C + D + G1 + G2 + N],
            [A + B + D + E + G1 + G2 + N],
            [B + C + N],
        ]
        index = 0

        self._display.fill(0)

        while index < len(numbers):
            self.animate([index], numbers[index], 0, auto_write)
            self._display.show()
            sleep(1)
            self._display.fill(0)
            sleep(0.5)

            index += 1

        sleep(1)
        self._display.fill(0)



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Basic example of drawing an image
# This example and library is meant to work with Adafruit CircuitPython API.
#
# This example is for use on (Linux) computers that are using CPython with
# Adafruit Blinka to support CircuitPython libraries. CircuitPython does
# not support PIL/pillow (python imaging library)!
#
# Author: Melissa LeBlanc-Williams
# License: Public Domain

# Import all board pins.
import board
import busio
from PIL import Image

# Import the HT16K33 LED matrix module.
from adafruit_ht16k33 import matrix

# Create the I2C interface.
i2c = busio.I2C(board.SCL, board.SDA)

# Create the matrix class.
# This creates a 16x8 matrix:
mtrx = matrix.Matrix16x8(i2c)
# Or this creates a 16x8 matrix backpack:
# mtrx = matrix.MatrixBackpack16x8(i2c)
# Or this creates a 8x8 matrix:
# mtrx = matrix.Matrix8x8(i2c)
# Or this creates a 8x8 bicolor matrix:
# mtrx = matrix.Matrix8x8x2(i2c)
# Finally you can optionally specify a custom I2C address of the HT16k33 like:
# mtrx = matrix.Matrix16x8(i2c, address=0x70)

if isinstance(mtrx, matrix.Matrix8x8x2):
    image = Image.open("squares-color.png")
elif isinstance(mtrx, matrix.Matrix16x8):
    image = Image.open("squares-mono-16x8.png")
else:
    image = Image.open("squares-mono-8x8.png")

# Clear the matrix
mtrx.fill(0)
mtrx.image(image)
